/*                     v3.0 - 10/Jan/2007
 * ============================================================================
 *
 *                        U    U   GGG    SSSS  TTTTT
 *                        U    U  G       S       T
 *                        U    U  G  GG   SSSS    T
 *                        U    U  G   G       S   T
 *                         UUU     GG     SSS     T
 *
 *                 ========================================
 *                  ITU-T - USER'S GROUP ON SOFTWARE TOOLS
 *                 ========================================
 *
 *
 *     =============================================================
 *     COPYRIGHT NOTE: This source code, and all of its derivations,
 *     is subject to the "ITU-T General Public License". Please have
 *     it  read  in    the  distribution  disk,   or  in  the  ITU-T
 *     Recommendation G.191 on "SOFTWARE TOOLS FOR SPEECH AND  AUDIO
 *     CODING STANDARDS".
 ** This code has  (C) Copyright by CNET Lannion A TSS/CMC **
 **    =============================================================
 **
 **
 **MODULE:         USER-LEVEL FUNCTIONS FOR THE UGST G.722 MODULE
 **
 **ORIGINAL BY:
 **Simao Ferraz de Campos Neto
 **COMSAT Laboratories                    Tel:    +1-301-428-4516
 **22300 Comsat Drive                     Fax:    +1-301-428-9287
 **Clarksburg MD 20871 - USA              E-mail: simao.campos@labs.comsat.com
 **
 **History:
 **History:
 **~~~~~~~~
 **14.Mar.95  v1.0       Released for use ITU-T UGST software package Tool
 **                   based on the CNET's 07/01/90 version 2.00
 **01.Jul.95  v2.0       Changed function declarations to work with many compilers;
 **                   reformated <simao@ctd.comsat.com>
 **10.Jan.07  v3.0       Updated with STL2005 v2.2 basic operators
 **                   <{balazs.kovesi,stephane.ragot}@orange-ftgroup.com>
 **Porting on LPDSP32 - Prithwee Narayana Bhat
 **24.Nov.15 - LPDSP32 optimizations by Ivo Coenen:
 **- (32-bit) int for Word16 type instead of (16-bit) short
 **- mult_long multiplications in qmf_tx and qmf_rx functions
 **- indexing instead of pointer incrementing inside loops to get dualld instructions
 **- using circular buffers for FIR delay lines
 **- static inline functions
 **- hardware saturation in qmf_tx and qmf_rx functions
 **- shifting inside limit function
 **- binary search in quantl function
 **- numerous minor optimizations
 **- input/output code is unsigned char type for easier handling
 **31.May.17 - additions by Dennis Mitchler:
 **- Integrating PLC algorithm
 **============================================================================
 */

#include "typedef.h"

#include "g722.h"
#include "g722_com.h"
#include "funcg722.h"

#include "g722_plc.h"
#include "mem_func.h"

void g722_reset_encoder (g722_state *encoder)
{
    Word16 xl, il;
    Word16 xh, ih;
    int j;

    xl = xh = 0;

    for (j = 0; j < 24; j++)
        chess_loop_count(24)
        {
            encoder->qmf_tx_delayx[j] = 0;
        }
        encoder->p_tx_delayx = encoder->qmf_tx_delayx;

    il = lsbcodecrs(encoder);
    ih = hsbcodecrs(encoder);
}

/* .................... end of g722_reset_encoder() ....................... */

Word32 g722_encode (short *incode, unsigned char *code, int read1,
                    g722_state *encoder, int bps)
{
    /* Encoder variables */
    Word16 xl, il;
    Word16 xh, ih;
    Word16 xin0, xin1;
    Word16 shift = 8 - bps;

    /* Auxiliary variables */
    int i;

    /* Divide sample counter by 2 to account for QMF operation */
    read1 = read1 >> 1;

    /* Main loop - never reset */

    for (i = 0; i < read1; i++)
        chess_loop_range(2, MAX_FR_SIZE)
        {
            xin1 = *incode++;
            xin0 = *incode++;

            /* Calculation of the synthesis QMF samples */
            qmf_tx(xin0, xin1, &xl, &xh, encoder);

            /* Call the upper and lower band ADPCM encoders */
            il = lsbcod(xl, encoder);
            ih = hsbcod(xh, encoder);

            /* Mount the output G722 codeword: bits 0 to 5 are the lower-band
             * portion of the encoding, and bits 6 and 7 are the upper-band
             * portion of the encoding */
            code[i] = (unsigned char)(ih << 6 | il) >> shift;
        }

        /* Return number of samples read divided by 2 (=number of codes generated) */
        return (read1);
}

/* .................... end of g722_encode() .......................... */

void g722_reset_decoder (g722_state *decoder)
{
    Word16 il, ih;
    Word16 rl, rh;
    int j;

    il = ih = 0;

    for (j = 0; j < 24; j++)
        chess_loop_count(24)
        {
            decoder->qmf_rx_delayx[j] = 0;
        }
        decoder->p_rx_delayx = decoder->qmf_rx_delayx;

    rl = lsbcodecrs(decoder);
    rh = hsbcodecrs(decoder);
}

/* .................... end of g722_reset_decoder() ....................... */

short g722_decode (unsigned char *code, short *outcode, int mode,
                   int read1, g722_state *decoder)
{
    /* Decoder variables */
    Word16 il, ih;
    Word16 rl, rh;
    Word16 xout1, xout2;
    Word16 shifted;
    Word16 shift = mode - 1;

    /* Auxiliary variables */
    int i;

    /* Decode - reset is never applied here */
    for (i = 0; i < read1; i++)
    {
        /* Separate the input G722 codeword: bits 0 to 5 are the lower-band
         * portion of the encoding, and bits 6 and 7 are the upper-band
         * portion of the encoding */
        shifted = (short)(code[i] << shift);
        il = shifted & 0x3F;    /* 6 bits of low SB */
        ih = (shifted >> 6) & 3;    /* 2 bits of high SB */

        /* Call the upper and lower band ADPCM decoders */
        rl = lsbdec(il, mode, decoder);
        rh = hsbdec(ih, decoder);

        /* Calculation of output samples from QMF filter */
        qmf_rx(rl, rh, &xout1, &xout2, decoder);

        /* Save samples in output vector */
        *outcode++ = xout1;
        *outcode++ = xout2;
    }

    /* Return number of samples generated */
    return (read1 << 1);
}

/* .................... end of g722_decode() .......................... */

/*----------------------------------------------------------------------
 * G722PLC_rem_dc(mem_in, mem_out, in)
 * DC removal filter * mem_in  (i/o) : filter memory
 * mem_out (i/o) : filter memory
 * in	     (i)   : input sample
 *----------------------------------------------------------------------*/
static inline Word16 G722PLC_rem_dc(Word16 *mem_in, Word16 *mem_out, Word16x in)
{
    Word16 out;
    Word16 temp;
    accum_t sum, mult1, mult2;

    /* accum_t tmp;
     * coef1 = 29212;
     * coef2 = 25656; */

    /* 25656 = 0.783 in Q15: 0 dB gain in pass band */

    /* temp = mult_r(in,29212);
     * out = add(temp, sub(mult_r(*mem_out, 25656), *mem_in));
     * *mem_in = temp;
     * *mem_out = out; */

    mult1 = fract_mult((Word32)in, 29212 << 16);
    mult2 = fract_mult(*mem_out, 25656 << 16);
    sum = mult1 + mult2 - to_accum(*mem_in);
    *mem_in = rnd_saturate(mult1);
    out = rnd_saturate(sum);

    /* printf("%02X%08X ",extract_ext(sum), extract_high(sum)); */
    *mem_out = out;
    return out;
}

int count = 0;
short G722PLC_decode(unsigned char *code, short *outcode, int mode, Word32 read1,
                     g722_state *decoder, void *plc_state)
{
    /* Decoder variables */
    Word16 il, ih;
    Word16 rl, rh;
    Word16 xout1, xout2;
    Word16 shifted;
    Word16 shift = mode - 1;

    /* Auxiliary variables */
    int i;
    Word16 ind_l, ind_h;
    Word16 *ptr_l;
    Word16 *ptr_h;
    G722PLC_STATE *state = (G722PLC_STATE *)plc_state;

    /* Word16 * filtmem, *filtptr; */
    Word16 weight0, weight1;
    Word16 tmp;
    Word16 count_crossfade;
    Word16 hpf_in, hpf_out;
    Word16 *b_mem_speech;
    Word16 *b_mem_speech_hb;
    Word16 *crossfade_buf;
    accum_t sum, mult1, mult2;
    Word16 coef1, coef2, dcrem_shift;

/* TO DO:  - to be done; + done; X won't be done
** store cross fader weights as 2 constant arrays in DMB instead of calculating them on the fly **DONE**
*- use 16 bits for mem_speech_hb to save on doing 16 bit left shifts in this function
*     > **still 32 bits, but only 16 LSBs are significant**
*  -  use 16 bits for mem_speech to save on doing 16 bit left shifts in this function
*  X  when storing (in g722_plc.c) crossfade_buf shift right by 16 bits to avoid doing it in this routine
*     > **code rewritten so shift is not required**
** support frame sizes < 10ms, i.e 2, 4, 5, and 7.5 ms  **DONE**
** -  move all 4 buffers from heap to using fixed arrays
** document that only frame sizes up to 20 ms supported
**
*/
    state->prev_bfi = 0;

    /* shift speech buffers */

    /* ind_l = sub(L_MEM_SPEECH, state->l_frame);
     * G722PLC_copy(&state->mem_speech[state->l_frame],state->mem_speech, ind_l); / *shift 10 ms* /
     * ind_h = sub(160, state->l_frame);
     * G722PLC_copy(&state->mem_speech_hb[state->l_frame],state->mem_speech_hb, ind_h); / *shift 10 ms* / */
    ptr_l = state->mem_speech;
    ptr_h = state->mem_speech_hb;

    /* increment counter */
    coef1 = 32768;
    coef2 = 0;
    dcrem_shift = 32;
    if (state->count_hpf < 32000)
    {
        coef1 = 29212;
        coef2 = 25656 << 16;
        dcrem_shift = 0;
    }
    state->count_hpf = min(32000, state->count_hpf + read1);    /* upper count after loop, 32000 = 4sec*/

    count_crossfade = state->count_crossfade;
    crossfade_buf = &state->crossfade_buf[count_crossfade];
    hpf_in  = state->mem_hpf_in;
    hpf_out = state->mem_hpf_out;
    b_mem_speech = state->b_mem_speech;
    b_mem_speech_hb = state->b_mem_speech_hb;

    for (i = 0; i < read1; i++)
    {
        /* Separate the input G722 codeword: bits 0 to 5 are the lower-band
         * portion of the encoding, and bits 6 and 7 are the upper-band
         * portion of the encoding */
        shifted = (short)(code[i] << shift);
        il = shifted & 0x3F;    /* 6 bits of low SB */
        ih = (shifted >> 6) & 3;    /* 2 bits of high SB */

        /* Call the upper and lower band ADPCM decoders */
        rl = lsbdec(il, mode, decoder);
        rh = hsbdec(ih, decoder);

        /* weight = min(32768, max(0, rnd_saturate(fract_mult((count_crossfade-19)<<15, 546<<16)))); */
        weight0 = G722PLC_xf_weight0[min(CROSSFADELEN - 1, count_crossfade)];
        weight1 = G722PLC_xf_weight1[min(CROSSFADELEN - 1, count_crossfade)];

        count_crossfade++;

        /* rl = rnd_saturate((fract_mult(rl, weight1) + fract_mult((*crossfade_buf++)>>16, 32768 - weight1))<<16); */
        rl = rnd_saturate(fract_mult_su(rl, weight0) + fract_mult_su((*crossfade_buf++), weight1));

        /* inline of DC remove filter */
        mult1 = fract_mult(rh << 16, coef1);
        mult2 = fract_mult(hpf_out, coef2);
        sum = mult1 + mult2 - to_accum(hpf_in << dcrem_shift);
        hpf_in = rnd_saturate(mult1);
        rh = rnd_saturate(sum);
        hpf_out = rh;

        /* copy lower and higher band sample */
        *ptr_l = rl << 16;
        *ptr_h = rh;

        ptr_l = cyclic_add(ptr_l, 1, b_mem_speech, L_MEM_SPEECH);
        ptr_h = cyclic_add(ptr_h, 1, b_mem_speech_hb, LEN_HB_MEM);

        /* Calculation of output samples from QMF filter */
        qmf_rx(rl, rh, &xout1, &xout2, decoder);

        /* Save samples in output vector */
        *outcode++ = xout1;
        *outcode++ = xout2;
    }
    state->count_crossfade = min(CROSSFADELEN, count_crossfade);
    state->mem_hpf_in  = hpf_in;
    state->mem_hpf_out = hpf_out;
    state->mem_speech    = ptr_l;
    state->mem_speech_hb = ptr_h;

    /* Return number of samples read */
    return (read1 << 1);
}

/* .................... end of g722_decode() .......................... */

#define DLT  decoder->dlt
#define PLT  decoder->plt
#define RLT  decoder->rlt
#define SL   decoder->sl
#define SZL  decoder->szl
#define DETL decoder->detl
#define NBL  decoder->nbl
#define DETH decoder->deth
#define NBH  decoder->nbh

void G722PLC_qmf_updstat (
    short *outcode,
    Word16 read1,      /* original type was 'short' */
    g722_state     *decoder,
    Word16 *lb_signal,    /* original type was 'short' */
    Word16 *hb_signal,    /* original type was 'short' */
    void *plc_state)
{
    Word16 rh;
    Word16 i;
    G722PLC_STATE *state = (G722PLC_STATE *)plc_state;
    Word16 *endLastOut;
    Word16 *firstFuture;

    /* Word16 * filtmem, *filtptr; */
    Word16 read2;
    Word16 xout1, xout2;
    int ptr;

    /* read2 = shl(read1,1);
     * filtmem = (Word16 *)calloc_(add(read2, 22), sizeof(Word16));
     * G722PLC_copy(&decoder->qmf_rx_delayx[2],&filtmem[read2], 22); / *load memory* /
     * filtptr = &filtmem[read2]; */

    ptr = ((int)decoder->p_rx_delayx - (int)&decoder->qmf_rx_delayx) / 4 - 2;
    if (ptr < 0)
    {
        ptr = 24 + ptr;
    }
    decoder->qmf_rx_delayx[ptr] = 0;
    decoder->qmf_rx_delayx[ptr + 1] = 0;
    for (i = 0; i < read1; i++)
        chess_loop_range(2, 20)
        {
            /* filter higher-band */
            rh = G722PLC_rem_dc(&state->mem_hpf_in, &state->mem_hpf_out, *hb_signal);

            /* calculate output samples of QMF filter */

            /* qmf_rx_buf (*lb_signal, rh, &filtptr, &outcode); */
            qmf_rx(*lb_signal >> 16, rh, &xout1, &xout2, decoder);

            /* printf("G722PLC_qmf_updstat: %3d %08X %08X %08X %08X %08X\n",i, *hb_signal>>16, *lb_signal>>16, rh,
             * xout1, xout2);
             * printf("G722PLC_qmf_updstat: %3d %8d  %8d %8d\n",i, *lb_signal, *hb_signal, rh); */

            /* / * Save samples in output vector * / */
            *outcode++ = xout1;
            *outcode++ = xout2;

            /* lb_signal++;
             * hb_signal++; */
            lb_signal = cyclic_add(lb_signal, 1, state->b_mem_speech, L_MEM_SPEECH);
            hb_signal = cyclic_add(hb_signal, 1, state->b_mem_speech_hb, LEN_HB_MEM);
        }

        /* G722PLC_copy(filtmem,&decoder->qmf_rx_delayx[2], 22); / *save memory* /
         * free_(filtmem); */

        /* reset G.722 decoder */

        /* endLastOut = &(state->mem_speech[L_MEM_SPEECH - 1]); */
        endLastOut = cyclic_add(state->mem_speech, L_MEM_SPEECH - 1, state->b_mem_speech, L_MEM_SPEECH);
    firstFuture = state->crossfade_buf;

    DLT[0] = DLT[1] = DLT[2] = DLT[3] = DLT[4] = DLT[5] = DLT[6] = 0;

    /* PLT[1] =  shr(endLastOut[0],1);
     * PLT[2] = shr(endLastOut[-1],1); */
    PLT[1] =  endLastOut[0] >> 17;
    PLT[2] = endLastOut[-1] >> 17;

    /* if (PLT[1]>0)
     * {
     * PLT[1] = 0x7FFF;
     * }
     * else
     * {
     * PLT[1] = 0x8001;
     * }
     * if (PLT[2]>0)
     * {
     * PLT[2] = 0x7FFF;
     * }
     * else
     * {
     * PLT[2] = 0x8001;
     * } */
    RLT[1] = endLastOut[0] >> 16;

    SL = firstFuture[0] >> 16;

    /* SZL = shr(firstFuture[0],1)>>16; */
    SZL = firstFuture[0] >> 17;

    /* change scale factors (to avoid overshoot) */

    /* NBH  = shr(NBH, 1); */
    NBH  = NBH >> 1;
    DETH = scaleh(NBH);

    /* reset G.722 decoder after muting */

    /* if(sub(state->count_att_hb, 160) > 0)
     * if(state->count_att_hb*state->inc_att > 160) */
    if (state->count_att_hb > state->limit_att_hb)
    {
        DETL = 32;
        NBL = 0;
        DETH = 8;
        NBH = 0;
    }
}

#undef DLT
#undef PLT
#undef RLT
#undef SL
#undef SZL
#undef DETL
#undef NBL
#undef DETH
#undef DH
#undef NBH

/* .................... end of G722PLC_qmf_updstat() .......................... */
