# ----------------------------------------------------------------------------
# Copyright (c) 2017 Semiconductor Components Industries, LLC (d/b/a
# ON Semiconductor), All Rights Reserved
#
# This code is the property of ON Semiconductor and may not be redistributed
# in any form without prior written permission from ON Semiconductor.
# The terms of use and warranty for this code are covered by contractual
# agreements between ON Semiconductor and the licensee.
#
# This is Reusable Code.
#
# -----------------------------------------------------------------------------
#

from argparse import ArgumentParser, FileType

from gdbDriver import gdbDriver
from fileio.binaryFile import binaryFile
from fileio.hexByteStream import hexByteStream
from fileio.hexWordStream import hexWordStream

if __name__ == '__main__':

    # The arguments to this are quite complex as it has been designed to allow
    # most things to be specified on the command line
    #
    parser = ArgumentParser(description = 'GDB Audio IO')
    
    # --exe <file>
    #    Defines the executable file that should be loaded to the CM3, this
    #    must be an ELF formatted executable with debugging information
    #    enabled
    #
    parser.add_argument('--exe', type=FileType('r'), required=True)
    
    # --gdb <file>
    #    Defines the path to the GDB executable to be used
    #    When running from within the RSL10 SDK, this is commonly
    #        ${eclipse_home}/../arm_tools/bin/arm-none-eabi-gdb.exe
    #
    parser.add_argument('--gdb', type=FileType('r'), required=True)
    
    # --gdbServer <file>
    #    Defines the path to the GDB server application.
    #    This would commonly be provided as part ofthe Segger JLink installation
    #    and can commonly be found in a folder of the form:
    #        C:\Program Files (x86)\SEGGER\JLink_Vxxxx\JLinkGDBServerCL.exe
    #
    parser.add_argument('--gdbServer', type=FileType('r'), required=True)
    
    # --jlinkId <id>
    #    Allows the specification of a known Jlink device, useful if there are
    #    more than one devices connected to the PC
    #
    parser.add_argument('--jlinkId')
    
    # --input <file>
    #    Defines the input data file
    #
    parser.add_argument('--input', type=FileType('rb'), required=True)

    # --golden <file>
    #    Defines the file which contains the golden results against which the
    #    output should be compared
    #
    parser.add_argument('--golden', type=FileType('rb'))
    
    # -- output <file>
    #    Defines a file to write the result encoded or decoded data
    #
    parser.add_argument('--output', type=FileType('wb'))
    
    # The following section defines different forms for the input data, only
    # one of these may be selected at a time
    #
    inTypes = parser.add_mutually_exclusive_group()
    inTypes.add_argument('--inBinary', action='store_const', dest='fileWrapperIn', const=binaryFile)
    inTypes.add_argument('--inHexBytes', action='store_const', dest='fileWrapperIn', const=hexByteStream)
    inTypes.add_argument('--inHexWords', action='store_const', dest='fileWrapperIn', const=hexWordStream)
    
    # The following section defines different forms for the golden data, only
    # one of these may be selected at a time
    #
    outTypes = parser.add_mutually_exclusive_group()
    outTypes.add_argument('--outBinary', action='store_const', dest='fileWrapperOut', const=binaryFile)
    outTypes.add_argument('--outHexBytes', action='store_const', dest='fileWrapperOut', const=hexByteStream)
    outTypes.add_argument('--outHexWords', action='store_const', dest='fileWrapperOut', const=hexWordStream)

    # Define the input and output packet sizes, this indicates the number of
    # elements to be read from the given input or golden file for each
    # transaction.
    #
    #    e.g. for G722 encoding, mode 1, with 160 samples per frame,
    #        inPacketSize would be 320
    #        outPacketSize would be 80
    #
    parser.add_argument('--inPacketSize', type=int, default=128)
    parser.add_argument('--outPacketSize', type=int, default=128)
    
    # Denote if we should encode or decode the data
    #
    encodeDecode = parser.add_mutually_exclusive_group()
    encodeDecode.add_argument('--encode', dest='encode', action='store_true', default=False)
    encodeDecode.add_argument('--decode', dest='encode', action='store_false', default=False)

    # Some codec specific items that can be used to control the behaviour of
    # the running code
    #
    #    Framesize is codec dependent, defines number of samples per frame
    #    Blocksize defines the number of samples in a subframe or block
    #    Sample rate defines the sampling rate for Celt
    #    Mode is used to determine the encoded bits for G722
    #    Packed indicates if packing should occur or not
    #    Stereo indicates we should run the test on two different channels,
    #        Only currently used for G722
    #
    parser.add_argument('--frameSize', type=int, default=160)
    parser.add_argument('--blockSize', type=int, default=4)
    parser.add_argument('--sampleRate', type=int, default=16000)
    parser.add_argument('--mode', type=int, default=1)
    parser.add_argument('--packed', action='store_true', default=False)
    parser.add_argument('--stereo', action='store_true', default=False)
    parser.add_argument('--precision', type=int, default=0)
    
    # Okay, parse the arguments as defined above
    args = parser.parse_args()

    # we don't want the debugger, jlink or executable to be open, so close them now
    args.exe.close()
    args.gdb.close()
    args.gdbServer.close()
    
    # we also want to wrap the data files so we can provide the content
    # in a standard way
    infile = args.fileWrapperIn(args.input)
    goldfile = args.fileWrapperOut(args.golden) if args.golden is not None else None
    
    # create and start the debugger, using default settings
    debugger = gdbDriver(args.gdb.name, args.gdbServer.name, args.exe.name, args.jlinkId, args.stereo)
    
    # define the packet sizes based on our arguments
    debugger.inPacketSize = args.inPacketSize
    debugger.outPacketSize = args.outPacketSize
    
    # setup the encoder parameters based on the incoming arguments
    debugger.encode = args.encode
    debugger.sampleRate = args.sampleRate
    debugger.frameSize = args.frameSize
    debugger.blockSize = args.blockSize
    debugger.mode = args.mode
    debugger.packed = args.packed
    debugger.precision = args.precision
    
    #We may wish to store the output files for later processing
    debugger.enableOutput(args.output)
    
    # connect to the debugger, compare the input data to the output and then
    # disconnect ... this will actually run the defined test
    debugger.connect()
    debugger.compare(infile, goldfile)
    debugger.disconnect()
    
